@@ -8,6 +8,7 @@ import h5py
 from . import kapre
 from . import tcn
 from . import models
+from typing import Dict, Callable, Any, List, Tuple
 
 
 class LossHistory(keras.callbacks.Callback):
@@ -37,51 +38,60 @@ class LossHistory(keras.callbacks.Callback):
         logging.info('trained ended')
 
 
-def load_model(file_trunk, model_dict, model_ext='_model.h5', from_epoch=False,
-               params_ext='_params.yaml', compile=True):
+def load_model(file_trunk: str, model_dict: Dict[str, Callable],
+               model_ext: str = '_model.h5',
+               params_ext: str = '_params.yaml',
+               compile: bool = True):
     """Load model with weights.
 
     First tries to load the full model directly using keras.models.load_model - this will likely fail for models with custom layers.
     Second, try to init model from parameters and then add weights...
 
     Args:
-        file_trunk ([type]): [description]
-        model_dict ([type]): [description]
+        file_trunk (str): [description]
+        model_dict (Dict[str, Callable): [description]
         model_ext (str, optional): [description]. Defaults to '_weights.h5'.
-        from_epoch ([type], optional): Deprecated [description]. Defaults to None.
         params_ext (str, optional): [description]. Defaults to '_params.yaml'.
         compile (bool, optional): [description]. Defaults to True.
 
     Returns:
-        [type]: [description]
+        keras.Model
     """
+
     try:
-        model = keras.models.load_model(file_trunk + model_ext,
+        model_filename = _download_if_url(file_trunk + model_ext)
+        model = keras.models.load_model(model_filename,
                                         custom_objects={'Spectrogram': kapre.time_frequency.Spectrogram,
                                                         'TCN': tcn.tcn_new.TCN})
     except (SystemError, ValueError, AttributeError):
         logging.debug('Failed to load model using keras, likely because it contains custom layers. Will try to init model architecture from code and load weights from `_model.h5` into it.', exc_info=False)
         logging.debug('', exc_info=True)
-        model = load_model_from_params(file_trunk, model_dict, weights_ext=model_ext, compile=compile)
+        model = load_model_from_params(file_trunk, model_dict, weights_ext=model_ext, params_ext=params_ext, compile=compile)
     return model
 
 
-def load_model_from_params(file_trunk, model_dict, weights_ext='_model.h5', params_ext='_params.yaml', compile=True):
+def load_model_from_params(file_trunk: str, model_dict: Dict[str, Callable],
+                           weights_ext: str = '_model.h5',
+                           params_ext: str = '_params.yaml',
+                           compile: bool = True):
     """Init architecture from code and load model weights into it. Helps with model loading issues across TF versions.
 
     Args:
-        file_trunk ([type]): [description]
+        file_trunk (str): [description]
         models_dict ([type]): [description]
         weights_ext (str, optional): [description]. Defaults to '_model.h5' (use weights from model file).
         params_ext (str, optional): [description]. Defaults to '_params.yaml'.
         compile (bool, optional): [description]. Defaults to True.
 
     Returns:
-        [type]: [description]
+        keras.Model
     """
-    params = load_params(file_trunk, params_ext)
+    params = load_params(file_trunk, params_ext=params_ext)
+
     model = model_dict[params['model_name']](**params)  # get the model - calls the function that generates a model with parameters
-    model.load_weights(file_trunk + weights_ext)
+    weights_filename = _download_if_url(file_trunk + weights_ext)
+    model.load_weights(weights_filename)
+
     if compile:
         # Compile with random standard optimizer and loss so we can use the model for prediction
         # Just re-compile the model if you want a particular optimizer and loss.
@@ -90,29 +100,31 @@ def load_model_from_params(file_trunk, model_dict, weights_ext='_model.h5', para
     return model
 
 
-def save_params(params, file_trunk, params_ext='_params.yaml'):
+def save_params(params: Dict[str, Any], file_trunk: str, params_ext: str = '_params.yaml'):
     """Save model/training parameters to yaml.
 
     Args:
-        params ([type]): [description]
-        file_trunk ([type]): [description]
+        params (Dict[str]): [description]
+        file_trunk (str): [description]
         params_ext (str, optional): [description]. Defaults to '_params.yaml'.
     """
     with open(file_trunk + params_ext, 'w') as f:
         yaml.dump(params, f)
 
 
-def load_params(file_trunk, params_ext='_params.yaml'):
+def load_params(file_trunk: str,
+                params_ext: str = '_params.yaml') -> Dict[str, Any]:
     """Load model/training parameters from yaml
 
     Args:
-        file_trunk ([type]): [description]
-        params_ext (str, optional): [description]. Defaults to '_params.yaml'.
+        file_trunk (str): [description]
+        params_ext (strs, optional): [description]. Defaults to '_params.yaml'.
 
-        Returns:
-        [type]: [description]
+    Returns:
+        Dict[str, Any]: Parameter dictionary
     """
-    with open(file_trunk + params_ext, 'r') as f:
+    filename = _download_if_url(file_trunk + params_ext)
+    with open(filename, 'r') as f:
         try:
             params = yaml.load(f, Loader=yaml.FullLoader)
         except AttributeError:
@@ -120,21 +132,43 @@ def load_params(file_trunk, params_ext='_params.yaml'):
     return params
 
 
-def load_model_and_params(model_save_name, model_dict=models.model_dict):
-    # load parameters and model
+def load_model_and_params(model_save_name, model_dict=models.model_dict) -> Tuple[keras.Model, Dict[str, Any]]:
+    """[summary]
+
+    Args:
+        model_save_name ([type]): [description]
+        model_dict ([type], optional): [description]. Defaults to models.model_dict.
+
+    Returns:
+        keras.Model, Dict[str, Any]: [description]
+    """
     params = load_params(model_save_name)
-    model = load_model(model_save_name, model_dict, from_epoch=False)
+    model = load_model(model_save_name, model_dict)
     return model, params
 
 
-def load_from(filename, datasets):
+def _download_if_url(url: str):
+    if not url.startswith('http'):
+        return url
+    else:
+        import urllib.request
+        import tempfile
+        from pathlib import Path
+        filename = url.split('/')[-1]  # get filename
+        tmpdir = tempfile.mkdtemp()
+        local_path = Path(tmpdir) / filename
+        urllib.request.urlretrieve(url, local_path)
+        return local_path
+
+
+def load_from(filename: str, datasets: List[str]):
     """Load datasets from h5 file.
 
     Args:
-        filename ([type]): [description]
-        datasets ([type]): [description]
+        filename (str)
+        datasets (List[str]): Names of the datasets (=keys) to load
 
-        Returns:
+    Returns:
         [type]: [description]
     """
     data = dict()
